'use strict';

const qs = require('querystring');
const Buffer = require('buffer').Buffer;
const Crypto = require('crypto');

function prepareBody(data) {
  if (!data) {
    return data
  }

  const newData = {};

  Object.keys(data).forEach(key => {
    if (data[key]) {
      newData[key] = data[key]
    }
  });

  return qs.stringify(newData).toString('utf-8')
}

function generateCryptoKeys(secretKey) {
  const iv = Buffer.allocUnsafe(16);
  const key = Crypto
    .createHash('sha256')
    .update(secretKey)
    .digest();

  key.copy(iv);

  return { key, iv }
}

const Utils = {

  sendMessage(accountSID, authToken, { phoneNumber, message, mediaUrl, senderPhoneNumber, statusCallback }) {
    return Backendless.Request
      .post(`https://api.twilio.com/2010-04-01/Accounts/${ accountSID }/Messages.json`)
      .set({
        'Authorization': `Basic ${ Buffer.from(`${ accountSID }:${ authToken }`).toString('base64') }`,
        'Content-Type' : 'application/x-www-form-urlencoded charset=utf-8'
      })
      .send(prepareBody({
        From          : senderPhoneNumber,
        To            : phoneNumber,
        Body          : message,
        MediaUrl      : mediaUrl,
        StatusCallback: statusCallback
      }))
  },

  generateVerificationCode() {
    let code = '';

    for (let i = 0; i < 5; i++) {
      code = code + Math.floor(Math.random() * 10);
    }

    return code;
  },

  encrypt(value, secretKey) {
    value = JSON.stringify(value);

    const { key, iv } = generateCryptoKeys(secretKey);

    const cipher = Crypto.createCipheriv('aes256', key, iv);

    return Buffer.concat([cipher.update(value), cipher.final()]).toString('hex');
  },

  decrypt(value, secretKey) {
    if (!value || !secretKey) {
      return null
    }

    const { key, iv } = generateCryptoKeys(secretKey);

    const encryptedText = Buffer.from(value, 'hex');
    const decipher = Crypto.createDecipheriv('aes256', key, iv);
    const data = Buffer.concat([decipher.update(encryptedText), decipher.final()]).toString();

    try {
      return JSON.parse(data);
    } catch (e) {
      throw new Error('Could not decode value')
    }
  },

};

module.exports = Utils;
